/**
 * Adds a default empty profile to the editor
 */
 function useTemplate(key) { 
   if(confirm("Are you sure you want to load this template?  The current content in the editor will be replaced.")) {
     _useTemplate(key);
   }
 }
 
 function _useTemplate(key) {
   templates[key](editor); 
 }

/**
 * Build a 2-tiered list of command links and configure the onClick event for
 * each link to insert the XML for that command.
 */
function setupCommands() {

  for(var key in commands) {
    if(typeof(commands[key]) == "string") {
      $('#command-list').append("<li><a href='#' rel='"+key+"'>" + key + "</a></li>");
    } else {
      var string = "<li><a href='#' class='command-header'>"+key+"</a><ul>";
      for(var key2 in commands[key]) {
        string += "<li><a href='#' rel='"+key+"."+key2+"'>" + key2 + "</a></li>";
      }
      string +="</ul></li>";
      $('#command-list').append(string);
    }
  }
  
  $('#command-list li ul li a').click(function(){
    var command = $(this).attr('rel').toString();
    var parts = command.split(".");
    var commandSet = commands;
    for(var key in parts) {
      key = parts[key];
      if(typeof(commandSet[key]) == "string") {
        command = commandSet[key];
      } else {
        commandSet = commandSet[key];
      }
    }
    insert(command);
    return false;
  });
}

function setupTemplates() {
  $('#template-list a').click(function(){
    useTemplate($(this).attr('key'));
    return false;
  });
}

/**
 * Build a 2-tiered list of command links and configure the onClick event for
 * each link to insert the XML for that command.
 */
function setupAttributes() {
	for(var command in attributes) {
		var markup = '<div class="attr-list '+command+'">';
		markup += "<strong>" + command + "</strong><br />";
		markup += "<ul>";
		for(var attribute in attributes[command]) {
			markup += "<li><a href='#' rel='" + command + ":" + attribute +   "' title='" + attributes[command][attribute]['description'] + "'>" + attribute + "</a></li>"; 
		}
		markup += "<li><a class='onfail' href='#' title='Continue execution when this command fails'>onfail</a></li>"; 
		
		markup += "</ul></div>";
	  $('#attr-list').append(markup);
  }

	$('.attr-list').hide();

	$('#attr-list a').click(function() {
		var lineNum = editor.getCursor().line;
		var line = editor.getLine(lineNum);
    
    if($(this).hasClass('onfail')) {
      insertInCurrentTag('onfail="true"');
    } else {
		  var parts = $(this).attr('rel').split(':');
		  var regex = new RegExp(parts[1] + "\s*=");
		  if(!line.match(regex)) { 
			  insertInCurrentTag(parts[1] + '="' + attributes[parts[0]][parts[1]]['default'] + '"');
		  }
    }
		return false;
	});
}

function checkCommand() {
	var lineNum = editor.getCursor().line;
	var line = editor.getLine(lineNum);
	var command = line.match(/<\w+/);
	if(command != null) {
		command = command.toString().substring(1);
		if(!$('.attr-list.' + command).is(':visible')) {
			$('.attr-list').hide();
			$('.attr-list.' + command).show();
		}
	} else {
		$('.attr-list').hide();
	}
}

function showAttributeList(command) {
	$('.attr-list.' + command).show();
}

function insertInCurrentTag(str) {
	editor.focus();
	var lineNum = editor.getCursor().line;
	var line = editor.getLine(lineNum);
  
  if(line.indexOf("</") != -1) return;

  var start = line.indexOf("/>");
  if(start == -1) start = line.indexOf(">");

  if(start != -1) {
		editor.setCursor(lineNum, start);
		editor.replaceRange(" " + str + " ", editor.getCursor());
  }
}

/**
 * Inserts a string into the current position in the editor
 */
function insert(str) {
  editor.focus();
  if(str.indexOf("\n") != -1) {
    var lines = str.split("\n");
    for(var key in lines) {
      insert($.trim(lines[key]));
    }
    return;
  }

  /**
   * If the current line has no visible characters, simply indent and insert the string.
   * If the current line is not empty, add the data to the next line
   */
   if($.trim(editor.getLine(editor.getCursor().line)) == "") {
    editor.setCursor(editor.getCursor().line, 0);
    indent();
    editor.replaceRange(str, editor.getCursor());
    indent();
  } 
  else {
    editor.replaceRange("\n", editor.getCursor()); // go to the next line
    indent();
    editor.replaceRange(str, editor.getCursor()); // insert the string and a new line character
    indent();
  } 
}

function insertBlankLine() {
  editor.replaceRange("\n", editor.getCursor()); // go to the next line
  indent();
  editor.replaceRange("\n", editor.getCursor()); // go to the next line
  indent();
}

function setupCopyPaste() {

  if(jQuery.browser.msie == true) {
    $('#copyProfile').click(function(){
      window.clipboardData.setData("Text",editor.getSelection());
      return false;
    });
  
    $('#paste').click(function(){
      var text = window.clipboardData.getData("Text");
      editor.replaceSelection(text);
      return false;
    });
  } else {
    $('#paste').remove();
    $('#copyProfile').remove();
  }
  
}

/**
 * Indent the current line
 */
function indent() {
  editor.indentLine(editor.getCursor().line);
}

/**
 * Start it up!
 */  
$(document).ready(function() {
  
  editor = CodeMirror.fromTextArea(document.getElementById("code"), {
    onKeyEvent: function(i, e) {
    //console.log(e.keyCode);
    if (e.keyCode == 32 && (e.ctrlKey || e.metaKey) && !e.altKey) {
        e.stop();
        console.log("calling startComplete");
        return startComplete();
      }
    },
    mode: { name: "customxml", htmlMode: true },
    tabMode : "shift",
    indentUnit : 2,
    onCursorActivity : checkCommand
  });
  
  
  
  setupTemplates();
  _useTemplate('default');
  setupCommands();
  setupAttributes();
  setupCopyPaste();
  
  $(".toggle-link").click(function(){
    $(this).parent().next().toggle();
    return false;
  });
  
  $("#command-list ul").hide();
  $("a.command-header").click(function(){
    $(this).next().toggle();
    return false;
  });

  $("#saveProfile").click(function() {
    var value = editor.getValue();
    console.log(value); 
    //window.external.saveProfileEditor(value);
    return false;
  });
});